"""
Module realize liveness mixin
"""

from typing import Awaitable, Dict, List, Optional, Union

from luna3.common.http_objs import BinaryImage
from luna3.common.luna_response import LunaResponse
from luna3.lunavl.mixins.liveness import _buildImagesPayload, _buildUrlsPayload

from .base import Backport3MixinStub


class LivenessMixin(Backport3MixinStub):
    """Methods to work with liveness API"""

    def predictLiveness(
        self,
        images: Optional[Union[BinaryImage, List[BinaryImage]]] = None,
        urls: Optional[List[str]] = None,
        meta: Optional[Dict] = None,
        xMeta: Optional[Dict] = None,
        aggregate: Optional[int] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Liveness API allows to predict liveness for given images.

        https://docs.idrnd.net/face/#facial-anti-spoofing-api

        Args:
            images: Single image or images list (supported formats "jpeg, png, tif, ppm, bmp").
            urls: list of urls with images
            meta: A dict with device information
                Fields:
                    os: A str, device os. Required
                    manufacture: A str, manufacture
                    model: A str, model
            xMeta: A dict with any additional information
            aggregate: Available values : 0, 1; whether to aggregate liveness estimations

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse`: results of predictions (failed and succeeded predictions)

        """
        if images:
            payload = _buildImagesPayload(images, self.multipartBoundary, meta, xMeta)
        elif urls:
            payload = _buildUrlsPayload(urls, meta, xMeta)
        else:
            raise ValueError("Either images or urls are required")

        queries = {"aggregate": aggregate}

        return self.makeAuthRequest(
            url=f"{self.baseUri}/liveness", method="POST", body=payload, queryParams=queries, **kwargs
        )
