"""
Module realize persons mixin
"""

from typing import Awaitable, Optional, Union

from luna3.common.luna_response import LunaResponse
from luna3.common.requests import RequestPayload

from .base import Backport3MixinStub


class PersonMixin(Backport3MixinStub):
    """Person Mixin to work with persons."""

    def getPersons(
        self,
        page: Optional[int] = 1,
        pageSize: Optional[int] = 10,
        userData: Optional[str] = None,
        externalId: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function gets all persons of the account. Result is a list of persons and number of persons for the account.
        Each person is represented by *person_id*, *user_data*, *linked_descriptors*, linked_lists*.

        Args:
            page: page number, positive.
            pageSize: number of results per page, positive.
            userData: user data or part of user data to search by it
            externalId: external id of the person, if it has its own mapping in external system

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with persons.

        """
        queries = {"page": page, "page_size": pageSize, "user_data": userData, "external_id": externalId}
        return self.makeAuthRequest(url=f"{self.baseUri}/storage/persons", method="GET", queryParams=queries, **kwargs)

    def createPerson(
        self, userData: Optional[str] = "", externalId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function creates a person with user data and/or external id. New person's *Id* is returned.

        Args:
            userData: user data for person
            externalId: external id of the person, if it has its own mapping in external system

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with created person id.

        """
        data = {"user_data": userData}
        if externalId is not None:
            data["external_id"] = externalId
        payload = RequestPayload.buildJson(body=data)
        return self.makeAuthRequest(url=f"{self.baseUri}/storage/persons", method="POST", body=payload, **kwargs)

    def getPerson(self, personId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function gets a person, which corresponds to *personId*.

        Args:
            personId: person *id*

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` json with person.

        """
        return self.makeAuthRequest(url=f"{self.baseUri}/storage/persons/{personId}", method="GET", **kwargs)

    def deletePerson(self, personId: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function removes a person, which corresponds *personId*.

        Args:
            personId: person *id*

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        return self.makeAuthRequest(url=f"{self.baseUri}/storage/persons/{personId}", method="DELETE", **kwargs)

    def patchPerson(
        self, personId: str, userData: Optional[str] = None, externalId: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        The function patches *user_data* and/or external id to the person.

        Args:
            personId: person *id*
            userData: user data for person
            externalId: external id of the person, if it has its own mapping in external system

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        data = {}
        if userData is not None:
            data["user_data"] = userData
        if externalId is not None:
            data["external_id"] = externalId
        payload = RequestPayload.buildJson(body=data)
        return self.makeAuthRequest(
            url=f"{self.baseUri}/storage/persons/{personId}", method="PATCH", body=payload, **kwargs
        )
