"""
Module realize websocket mixin
"""

from contextlib import contextmanager
from numbers import Real
from typing import List, Optional, Type, Union

import aiohttp

from luna3.common.ws_base import WSBase
from luna3.lunavl.mixins.base import BaseLunaHttpClient


class WSMixin(WSBase):
    """Websocket Mixin to work with luna-backport4 ws subscription."""

    # auth header; real method is defined above (BaseLunaHttpClient)
    getAuthHeader: Type[BaseLunaHttpClient.getAuthHeader]

    def wsConnect(  # pylint: disable=arguments-differ,arguments-renamed
        self,
        handlerIds: Optional[List[str]] = None,
        sources: Optional[List[str]] = None,
        topSimilarFaceLists: Optional[List[str]] = None,
        topSimilarFaceSimilarityGte: Optional[float] = None,
        topSimilarFaceSimilarityLt: Optional[float] = None,
        ageLt: Optional[int] = None,
        ageGte: Optional[int] = None,
        gender: Optional[int] = None,
        ethnicGroups: Optional[List[int]] = None,
        tags: Optional[List[str]] = None,
        accountId: Optional[str] = None,
        login: Optional[str] = None,
        password: Optional[str] = None,
        token: Optional[str] = None,
        asyncRequest: Optional[bool] = None,
        messageReceiveTimeout: Optional[Real] = None,
    ) -> Union[aiohttp.client._WSRequestContextManager, contextmanager]:  # pylint: disable=W0212
        """
        Create ws-connection to luna-sender

        Args:
            accountId: account id to connect to
            handlerIds: list of handler ids
            ageLt: [ 0 .. 100 ] - upper bound for reference age
            ageGte: [ 0 .. 100 ] - lower including bound for reference age
            gender: Gender. 1 - male, 0 - female
            ethnicGroups: Array of integer (ethnicities_filter) - Items Enum:1 2 3 4
                         ( AfricanAmerican - 1; Indian - 2; Asian - 3; Caucasian - 4)
            tags: users tags
            topSimilarFaceLists: top similar face list ids
            topSimilarFaceSimilarityGte: top similar face similarity lower including boundary
            topSimilarFaceSimilarityLt: top similar face similarity upper excluding boundary
            sources: list of source
            asyncRequest: execution in asynchronous mode, disabled by default
            messageReceiveTimeout: message receive timeout (time between start waiting message and message appearance)
        Returns:
            context manager with connection to luna-sender

        >>> import asyncio
        >>> from luna3.lunavl.httpclient import LunaHttpClient
        >>> senderAPI = LunaHttpClient()
        >>> def syncws():
        ...     with senderAPI.connect() as ws:
        ...         for _ in range(3):
        ...             resp = ws.recv()
        ...             print(resp)
        >>> async def asyncws():
        ...     async with senderAPI.connect(asyncRequest=True) as ws:
        ...         count = 0
        ...         async for resp in ws:
        ...             count += 1
        ...             print(resp.data)
        ...             if count > 2:
        ...                 await ws.close()
        ...                 raise SystemExit(0)
        >>> syncws()
        'some three messages'
        >>> asyncio.run(asyncws())
        'another three messages'
        """
        queryParams = {
            "age__lt": ageLt,
            "age__gte": ageGte,
            "gender": gender,
            "ethnic_groups": ethnicGroups,
            "tags": tags,
            "sources": sources,
            "handler_ids": handlerIds,
            "top_similar_face_lists": topSimilarFaceLists,
            "top_similar_face_similarity__gte": topSimilarFaceSimilarityGte,
            "top_similar_face_similarity__lt": topSimilarFaceSimilarityLt,
        }
        headers = self.getAuthHeader(accountId=accountId, login=login, password=password, token=token)

        return super().wsConnect(
            headers=headers, queryParams=queryParams, messageTimeout=messageReceiveTimeout, asyncRequest=asyncRequest
        )
