"""
Module contains backward compatible match objects for luna-remote-sdk
"""

from dataclasses import dataclass
from typing import Literal, Optional, Union

from luna3.common.http_objs import BoundingBox, FaceQuality
from luna3.events.http_objs import BaseClass
from luna3.public.common import PeopleCount


@dataclass
class LivenessParams(BaseClass):
    """Liveness settings"""

    qualityThreshold: Optional[float] = None
    scoreThreshold: Optional[float] = None


@dataclass
class DeepfakeParams(BaseClass):
    """Deepfake settings"""

    realThreshold: float = None
    mode: Optional[int] = None


@dataclass
class EstimatorsParams(BaseClass):
    """Estimation parameters"""

    livenessParams: LivenessParams = None
    faceDescriptorVersion: Optional[int] = None
    bodyDescriptorVersion: Optional[int] = None
    deepfakeParams: DeepfakeParams = None


@dataclass
class Image(BaseClass):
    """Image for estimation"""

    body: bytes
    filename: str
    imageType: Optional[int] = 0
    faceDetections: Optional[list[BoundingBox]] = None
    bodyDetections: Optional[list[BoundingBox]] = None

    def asDict(self):
        image = super().asDict()
        if self.faceDetections:
            image["face_detections"] = [detection.asDict() for detection in self.faceDetections]
        if self.bodyDetections:
            image["body_detections"] = [detection.asDict() for detection in self.bodyDetections]
        return image


@dataclass
class FaceDetectionFilters(BaseClass):
    """Filters for face detection"""

    yawThreshold: Optional[float] = None
    pitchThreshold: Optional[float] = None
    rollThreshold: Optional[float] = None
    livenessStates: Optional[list[int]] = None
    maskStates: Optional[list[int]] = None
    scoreThreshold: Optional[float] = None
    deepfakeStates: Optional[list[int]] = None


@dataclass
class Filters(BaseClass):
    """Estimation filters"""

    faceDetectionFilters: FaceDetectionFilters


@dataclass
class Targets(BaseClass):
    """Estimation targets"""

    exif: int = None
    peopleCount: PeopleCount = None
    faceDetection: int = None
    faceWarp: int = None
    faceLandmarks5: int = None
    faceLandmarks68: int = None
    gaze: int = None
    headPose: int = None
    eyes: int = None
    mouthAttributes: int = None
    faceWarpQuality: int = None
    emotions: int = None
    faceDescriptor: int = None
    basicAttributes: int = None
    mask: int = None
    glasses: int = None
    liveness: int = None
    bodyWarp: int = None
    bodyDetection: int = None
    bodyDescriptor: int = None
    bodyAttributes: int = None
    upperBody: int = None
    accessories: int = None
    lowerBody: int = None
    faceQuality: FaceQuality = None
    deepfake: int = None

    def asDict(self) -> dict:
        """
        Recursively cast class to dict.

        Returns:
            self as dict without empty fields
        """
        res = super().asDict()
        if peopleCount := res.get("people_count"):
            res["people_count"] = peopleCount.asDict()
        if faceQuality := res.get("face_quality"):
            res["face_quality"] = faceQuality.asDict()
        return res


@dataclass
class Params(BaseClass):
    """Parameters for performing estimation request"""

    targets: Targets
    useExifInfo: Optional[int] = None
    filters: Optional[Filters] = None
    estimatorsParams: Optional[EstimatorsParams] = None
    aggregate: Optional[int] = None
    multifacePolicy: Optional[int] = None


@dataclass
class Estimator(BaseClass):
    """Estimator request schema"""

    params: Params
    images: list[Image]

    def asDict(self, noBBox: bool = False) -> dict:
        """
        Get dict

        Args:
            noBBox: no bounding box for warped images flag

        Returns:
            estimator
        """
        estimator = super().asDict()
        if noBBox:
            for image in estimator["images"]:
                if image["image_type"] in (1, 2):
                    image.pop("face_detections", None)
                    image.pop("body_detections", None)
        return estimator


@dataclass
class ROI(BaseClass):
    """
    Region of interest detection coordinates Model

    Attributes:
        x: Region of interest *x* coordinate
        y: Region of interest *y* coordinate
        height: Region of interest height
        width: Region of interest width
    """

    mode: Literal["abs", "percent"]
    x: Union[int, float]  # pylint: disable-msg=C0103
    y: Union[int, float]  # pylint: disable-msg=C0103
    height: Union[int, float]
    width: Union[int, float]


@dataclass
class Rate(BaseClass):
    """Rate"""

    period: int
    unit: Literal["second", "frame"]


class ImageRetainPolicy(BaseClass):
    """Image retain configuration"""

    # image quality in relative scale from 0 to 1
    quality: Optional[float] = None
    # image max size, neither the width nor the height will exceed this value
    maxSize: Optional[int] = None
    # image mimetype: JSON, or PNG
    mimetype: Optional[str] = None


@dataclass
class AnalyticsCommonParams(BaseClass):
    """Base analytics parameters"""

    # estimator region of interest
    roi: Optional[ROI] = None
    # probe count
    probeCount: Optional[int] = None
    # estimation rate
    rate: Optional[Rate] = None
    # image retain configuration
    imageRetainPolicy: Optional[ImageRetainPolicy] = None


@dataclass
class PeopleCountAnalyticsParams(AnalyticsCommonParams):
    """People count analytics parameters"""

    peopleCountThreshold: Optional[int] = None


@dataclass
class HumanTrackingAnalyticsParams(AnalyticsCommonParams):
    """Human tracking analytics parameters"""

    @dataclass
    class Tracking(BaseClass):
        """Human tracking configuration"""

        detectorType: Optional[Literal["face", "body", "human"]] = None
        bodyReid: Optional[int] = None

    @dataclass
    class Shots(BaseClass):
        """Configuration for choosing detections for attributes estimation"""

        count: Optional[int] = None
        filters: Optional[dict] = None

    # tracking configuration
    tracking: Optional[Tracking] = None
    # face shots configuration
    faceSamples: Optional[Shots] = None
    # body shots configuration
    bodySamples: Optional[Shots] = None


@dataclass
class VideoSDKAnalytics(BaseClass):
    """People count analytics"""

    #  analytics name
    name: Literal["people_count", "human_tracking"]
    #  analytics targets
    targets: Optional[list[str]] = None
    #  analytics params
    parameters: Union[PeopleCountAnalyticsParams, HumanTrackingAnalyticsParams, None] = None


@dataclass
class Orientation(BaseClass):
    """Video orientation info"""

    angle: Literal[0, 90, 180, 270]


@dataclass
class VideoSDKProperties(BaseClass):
    """Video url"""

    url: str
    orientation: Optional[Orientation] = None


@dataclass
class VideoSDKInput(BaseClass):
    """Video url"""

    video: VideoSDKProperties
    analytics: list[VideoSDKAnalytics]
